"""Chat channels module with plugin architecture."""

from joyhousebot.channels.base import BaseChannel
from joyhousebot.channels.manager import ChannelManager

__all__ = ["BaseChannel", "ChannelManager"]
